@@ -1,13 +1,15 @@
-#!/usr/bin/env python2.3
+#!/usr/bin/env python2.4
+from basetest import BaseTest
 import sys, tempfile, os, shutil
 from StringIO import StringIO
 import unittest, signal
-from logging import getLogger, DEBUG, INFO
+from logging import getLogger, DEBUG, INFO, WARN
 #getLogger().setLevel(DEBUG)
 
 sys.path.insert(0, '..')
 
-from zeroinstall.injector import model, basedir, autopolicy, gpg, iface_cache, download
+from zeroinstall.injector import model, basedir, autopolicy, gpg, iface_cache, download, reader, trust
+from zeroinstall.zerostore import Store; Store._add_with_helper = lambda *unused: False
 import data
 
 import server
@@ -19,38 +21,24 @@ class Reply:
 	def readline(self):
 		return self.reply
 
-class TestDownload(unittest.TestCase):
+class TestDownload(BaseTest):
 	def setUp(self):
-		self.config_home = tempfile.mktemp()
-		self.cache_home = tempfile.mktemp()
-		os.environ['XDG_CONFIG_HOME'] = self.config_home
-		os.environ['XDG_CACHE_HOME'] = self.cache_home
-		os.environ['XDG_CACHE_DIRS'] = ''
-		reload(basedir)
+		BaseTest.setUp(self)
 
-		os.mkdir(self.config_home, 0700)
-		os.mkdir(self.cache_home, 0700)
-		if os.environ.has_key('DISPLAY'):
-			del os.environ['DISPLAY']
-		self.gnupg_home = tempfile.mktemp()
-		os.environ['GNUPGHOME'] = self.gnupg_home
-		os.mkdir(self.gnupg_home, 0700)
 		stream = tempfile.TemporaryFile()
 		stream.write(data.thomas_key)
 		stream.seek(0)
 		gpg.import_key(stream)
-		iface_cache.iface_cache.__init__()
-		download._downloads = {}
 		self.child = None
+
+		trust.trust_db.watchers = []
 	
 	def tearDown(self):
+		BaseTest.tearDown(self)
 		if self.child is not None:
 			os.kill(self.child, signal.SIGTERM)
 			os.waitpid(self.child, 0)
 			self.child = None
-		shutil.rmtree(self.config_home)
-		shutil.rmtree(self.cache_home)
-		shutil.rmtree(self.gnupg_home)
 	
 	def testRejectKey(self):
 		old_out = sys.stdout
@@ -90,28 +78,38 @@ class TestDownload(unittest.TestCase):
 		old_out = sys.stdout
 		try:
 			from zeroinstall.injector import cli
-			from zeroinstall.injector.trust import trust_db
+			import logging
+
+			rootLogger = getLogger()
+			rootLogger.disabled = True
+			try:
+				try:
+					cli.main(['--import', '-v', 'NO-SUCH-FILE'])
+					assert 0
+				except model.SafeException, ex:
+					assert 'NO-SUCH-FILE' in str(ex)
+			finally:
+				rootLogger.disabled = False
+				rootLogger.setLevel(WARN)
+
+			hello = iface_cache.iface_cache.get_interface('http://localhost:8000/Hello')
+			self.assertEquals(0, len(hello.implementations))
+
 			sys.stdout = StringIO()
 			self.child = server.handle_requests('6FCF121BE2390E0B.gpg', 'HelloWorld.tgz')
 			sys.stdin = Reply("Y\n")
-			try:
-				assert not trust_db.is_trusted('DE937DD411906ACF7C263B396FCF121BE2390E0B')
-				try:
-					cli.main(['--import', 'Hello'])
-					assert 0
-				except SystemExit, ex:
-					assert ex.code == 0
-				assert trust_db.is_trusted('DE937DD411906ACF7C263B396FCF121BE2390E0B')
-				# Shouldn't need to prompt the second time
-				sys.stdin = None
-				try:
-					cli.main(['--import', 'Hello'])
-					assert 0
-				except SystemExit, ex:
-					assert ex.code == 0
-			except model.SafeException, ex:
-				if "HelloWorld/Missing" not in str(ex):
-					raise ex
+
+			assert not trust.trust_db.is_trusted('DE937DD411906ACF7C263B396FCF121BE2390E0B')
+			cli.main(['--import', 'Hello'])
+			assert trust.trust_db.is_trusted('DE937DD411906ACF7C263B396FCF121BE2390E0B')
+
+			# Check we imported the interface after trusting the key
+			reader.update_from_cache(hello)
+			self.assertEquals(1, len(hello.implementations))
+
+			# Shouldn't need to prompt the second time
+			sys.stdin = None
+			cli.main(['--import', 'Hello'])
 		finally:
 			sys.stdout = old_out
 	
@@ -147,6 +145,22 @@ class TestDownload(unittest.TestCase):
 		finally:
 			sys.stdout = old_out
 
+	def testSymlink(self):
+		old_out = sys.stdout
+		try:
+			sys.stdout = StringIO()
+			self.child = server.handle_requests(('HelloWorld.tar.bz2', 'HelloSym.tgz'))
+			policy = autopolicy.AutoPolicy(os.path.abspath('RecipeSymlink.xml'), download_only = False)
+			try:
+				policy.download_and_execute([])
+				assert False
+			except model.SafeException, ex:
+				if 'Attempt to unpack dir over symlink "HelloWorld"' not in str(ex):
+					raise ex
+			self.assertEquals(None, basedir.load_first_cache('0install.net', 'implementations', 'main'))
+		finally:
+			sys.stdout = old_out
+
 	def testAutopackage(self):
 		old_out = sys.stdout
 		try:
