@@ -1,8 +1,11 @@
+import zeroinstall
 from zeroinstall.injector.model import *
 from zeroinstall.injector.iface_cache import iface_cache
 from zeroinstall.injector import writer, namespaces, gpg
+
 import gtk, sys, os
 import sets	# Note: for Python 2.3; frozenset is only in Python 2.4
+from logging import warn
 
 import help_box
 from dialog import Dialog
@@ -42,6 +45,32 @@ def open_in_browser(link):
 			os._exit(1)
 	os.waitpid(child, 0)
 
+def have_source_for(interface):
+	# Note: we don't want to actually fetch the source interfaces at
+	# this point, so we check whether:
+	# - We have a feed of type 'src' (not fetched), or
+	# - We have a source implementation in a regular feed
+	have_src = False
+	for f in interface.feeds:
+		if f.machine == 'src':
+			return True
+	# Don't have any src feeds. Do we have a source implementation
+	# as part of a regular feed?
+	impls = interface.implementations.values()
+	for f in policy.usable_feeds(interface):
+		try:
+			feed_iface = iface_cache.get_interface(f.uri)
+			if feed_iface.implementations:
+				impls.extend(feed_iface.implementations.values())
+		except zeroinstall.NeedDownload:
+			pass	# OK, will get called again later
+		except Exception, ex:
+			warn("Failed to load feed '%s': %s", f.uri, str(ex))
+	for x in impls:
+		if x.machine == 'src':
+			return True
+	return False
+
 class Description(gtk.ScrolledWindow):
 	def __init__(self):
 		gtk.ScrolledWindow.__init__(self, None, None)
@@ -95,6 +124,13 @@ class Description(gtk.ScrolledWindow):
 		if interface.last_checked:
 			buffer.insert(iter, '\nLast checked: %s' % time.ctime(interface.last_checked))
 
+		if interface.last_check_attempt:
+			if interface.last_checked and interface.last_checked >= interface.last_check_attempt:
+				pass	# Don't bother reporting successful attempts
+			else:
+				buffer.insert(iter, '\nLast check attempt: %s (failed or in progress)' %
+						time.ctime(interface.last_check_attempt))
+
 		buffer.insert_with_tags(iter, '\n\nDescription\n', heading_style)
 
 		paragraphs = [format_para(p) for p in (interface.description or "-").split('\n\n')]
@@ -102,41 +138,37 @@ class Description(gtk.ScrolledWindow):
 		buffer.insert(iter, '\n\n'.join(paragraphs))
 		buffer.insert(iter, '\n')
 
-		if hasattr(interface, 'get_metadata'):
-			need_gap = True
-			for x in interface.get_metadata(namespaces.XMLNS_IFACE, 'homepage'):
-				if need_gap:
-					buffer.insert(iter, '\n')
-					need_gap = False
-				buffer.insert(iter, 'Homepage: ')
-				buffer.insert_with_tags(iter, '%s\n' % x.content, self.link_style)
-
-		if hasattr(iface_cache, 'get_cached_signatures'):
-			buffer.insert_with_tags(iter, '\nSignatures\n', heading_style)
-			sigs = iface_cache.get_cached_signatures(interface.uri)
-			if sigs:
-				for sig in sigs:
-					if isinstance(sig, gpg.ValidSig):
-						name = '<unknown>'
-						if hasattr(sig, 'get_details'):
-							details = sig.get_details()
-							for item in details:
-								if item[0] in ('pub', 'uid') and len(item) > 9:
-									name = item[9]
-									break
-						buffer.insert_with_tags(iter, 'Valid signature by "%s"\n- Dated: %s\n- Fingerprint: %s\n' %
-								(name, time.ctime(sig.get_timestamp()), sig.fingerprint))
-						if not sig.is_trusted():
-							if interface.uri.startswith('/'):
-								buffer.insert_with_tags(iter, 'WARNING: This key is not in the trusted list\n')
-							else:
-								buffer.insert_with_tags(iter, 'WARNING: This key is not in the trusted list (either you removed it, or '
-												'you trust one of the other signatures)\n')
-					else:
-						buffer.insert_with_tags(iter, '%s\n' % sig)
-			else:
-				buffer.insert_with_tags(iter, 'No signature information (old style interface or out-of-date cache)\n')
+		need_gap = True
+		for x in interface.get_metadata(namespaces.XMLNS_IFACE, 'homepage'):
+			if need_gap:
+				buffer.insert(iter, '\n')
+				need_gap = False
+			buffer.insert(iter, 'Homepage: ')
+			buffer.insert_with_tags(iter, '%s\n' % x.content, self.link_style)
 
+		buffer.insert_with_tags(iter, '\nSignatures\n', heading_style)
+		sigs = iface_cache.get_cached_signatures(interface.uri)
+		if sigs:
+			for sig in sigs:
+				if isinstance(sig, gpg.ValidSig):
+					name = '<unknown>'
+					details = sig.get_details()
+					for item in details:
+						if item[0] in ('pub', 'uid') and len(item) > 9:
+							name = item[9]
+							break
+					buffer.insert_with_tags(iter, 'Valid signature by "%s"\n- Dated: %s\n- Fingerprint: %s\n' %
+							(name, time.ctime(sig.get_timestamp()), sig.fingerprint))
+					if not sig.is_trusted():
+						if interface.uri.startswith('/'):
+							buffer.insert_with_tags(iter, 'WARNING: This key is not in the trusted list\n')
+						else:
+							buffer.insert_with_tags(iter, 'WARNING: This key is not in the trusted list (either you removed it, or '
+											'you trust one of the other signatures)\n')
+				else:
+					buffer.insert_with_tags(iter, '%s\n' % sig)
+		else:
+			buffer.insert_with_tags(iter, 'No signature information (old style interface or out-of-date cache)\n')
 
 class Feeds(gtk.VPaned):
 	URI = 0
@@ -174,7 +206,7 @@ class Feeds(gtk.VPaned):
 		add_local_feed_button = dialog.MixedButton(_('Add Local Feed...'), gtk.STOCK_ADD, 0.0)
 		add_local_feed_button.connect('clicked', lambda b: add_local_feed(interface))
 		tips.set_tip(add_local_feed_button,
-			_('If you have another implementation of this interface (e.g., a '
+			_('If you have another implementation of this interface (e.g. a '
 			  'CVS checkout), you can add it to the list by registering the XML '
 			  'feed file that came with it.'))
 		buttons_vbox.add(add_local_feed_button)
@@ -228,11 +260,17 @@ class Feeds(gtk.VPaned):
 		return out
 
 	def sel_changed(self, sel):
-		model, iter = sel.get_selected()
-		if not iter: return	# build in progress
-		iface = model[iter][Feeds.URI]
-		self.remove_feed_button.set_sensitive(iface != self.interface.uri)
-		self.description.set_details(policy.get_interface(iface))
+		model, miter = sel.get_selected()
+		if not miter: return	# build in progress
+		iface = model[miter][Feeds.URI]
+		# Only enable removing user_override feeds
+		enable_remove = False
+		for x in self.interface.feeds:
+			if x.uri == iface:
+				if x.user_override:
+					enable_remove = True
+		self.remove_feed_button.set_sensitive( enable_remove )
+		self.description.set_details(iface_cache.get_interface(iface))
 	
 	def updated(self):
 		new_lines = self.build_model()
@@ -249,7 +287,7 @@ class Properties(Dialog):
 	interface = None
 	use_list = None
 
-	def __init__(self, interface):
+	def __init__(self, interface, show_versions = False):
 		Dialog.__init__(self)
 		self.interface = interface
 		self.set_title('Interface ' + interface.get_name())
@@ -266,8 +304,6 @@ class Properties(Dialog):
 		def response(dialog, resp):
 			if resp == gtk.RESPONSE_CANCEL:
 				self.destroy()
-			#elif resp == 1:
-			#	policy.begin_iface_download(interface, True)
 			elif resp == gtk.RESPONSE_HELP:
 				properties_help.display()
 		self.connect('response', response)
@@ -291,35 +327,12 @@ class Properties(Dialog):
 		self.connect('destroy', lambda s: policy.watchers.remove(updated))
 		policy.watchers.append(updated)
 		self.shade_compile()
+
+		if show_versions:
+			notebook.next_page()
 	
 	def shade_compile(self):
-		# Note: we don't want to actually fetch the source interfaces at
-		# this point, so we check whether:
-		# - We have a feed of type 'src' (not fetched), or
-		# - We have a source implementation in a regular feed
-		have_src = False
-		for f in self.interface.feeds:
-			if f.machine == 'src':
-				have_src = True
-				break
-		if have_src is False:
-			# Don't have any src feeds. Do we have a source implementation
-			# as part of a regular feed?
-			impls = self.interface.implementations.values()
-			for f in policy.usable_feeds(self.interface):
-				try:
-					feed_iface = policy.get_interface(f.uri)
-					if feed_iface.implementations:
-						impls.extend(feed_iface.implementations.values())
-				except NeedDownload:
-					pass	# OK, will get called again later
-				except Exception, ex:
-					warn("Failed to load feed '%s': %s", f.uri, str(ex))
-			for x in impls:
-				if x.machine == 'src':
-					have_src = True
-					break
-		self.compile_button.set_sensitive(have_src)
+		self.compile_button.set_sensitive(have_source_for(self.interface))
 	
 	def update_list(self):
 		impls = policy.get_ranked_implementations(self.interface)
@@ -419,10 +432,10 @@ def add_remote_feed(parent, interface):
 				url = entry.get_text()
 				if not url:
 					raise SafeException(_('Enter a URL'))
-				iface = policy.get_interface(url)
+				iface = iface_cache.get_interface(url)
 				policy.begin_iface_download(iface) # Force a refresh
 				d.set_sensitive(False)
-				policy.add_dl_callback(url, lambda: download_done(iface))
+				policy.handler.add_dl_callback(url, lambda: download_done(iface))
 			except SafeException, ex:
 				error(str(ex))
 		else:
@@ -439,27 +452,16 @@ def add_local_feed(interface):
 		from zeroinstall.injector import reader
 		feed = sel.get_filename()
 		try:
-			if hasattr(policy, 'get_feed_targets'):
-				feed_targets = policy.get_feed_targets(feed)
-				if interface not in feed_targets:
-					raise Exception("Not a valid feed for '%s'; this is a feed for:\n%s" %
-							(interface.uri,
-							'\n'.join([f.uri for f in feed_targets])))
-				if interface.get_feed(feed):
-					dialog.alert(None, 'This feed is already registered.')
-				else:
-					interface.feeds.append(Feed(feed, user_override = True, arch = None))
+			feed_targets = policy.get_feed_targets(feed)
+			if interface not in feed_targets:
+				raise Exception("Not a valid feed for '%s'; this is a feed for:\n%s" %
+						(interface.uri,
+						'\n'.join([f.uri for f in feed_targets])))
+			if interface.get_feed(feed):
+				dialog.alert(None, 'This feed is already registered.')
 			else:
-				doc = minidom.parse(feed)
-				uri = doc.documentElement.getAttribute('uri')
-				if not uri:
-					raise Exception("Missing uri attribute in interface file '%s'" % feed)
-				if uri != interface.uri:
-					raise Exception("Feed is for interface '%s', not '%s'" %
-							(uri, interface.uri))
-				if feed in interface.feeds:
-					raise Exception("Feed is already registered")
-				interface.feeds.append(feed)
+				interface.feeds.append(Feed(feed, user_override = True, arch = None))
+
 			writer.save_interface(interface)
 			sel.destroy()
 			reader.update_from_cache(interface)
@@ -471,11 +473,11 @@ def add_local_feed(interface):
 	sel.cancel_button.connect('clicked', lambda b: sel.destroy())
 	sel.show()
 	
-def edit(interface):
+def edit(interface, show_versions = False):
 	assert isinstance(interface, Interface)
 	if interface in _dialogs:
 		_dialogs[interface].destroy()
-	_dialogs[interface] = Properties(interface)
+	_dialogs[interface] = Properties(interface, show_versions)
 	_dialogs[interface].show()
 
 properties_help = help_box.HelpBox("Injector Properties Help",
