@@ -26,13 +26,17 @@ class AutoPolicy(policy.Policy):
 		self.dry_run = dry_run
 
 	def need_download(self):
-		"""Decide whether we need to download anything (but don't do it!)"""
+		"""Decide whether we need to download anything (but don't do it!)
+		@return: true if we MUST download something (interfaces or implementations)
+		@rtype: bool
+		@postcondition: if we return False, self.stale_feeds contains any feeds which SHOULD be updated
+		"""
 		old = self.allow_downloads
 		self.allow_downloads = False
 		try:
 			try:
-				self.recalculate()
-				debug("Recalculated: ready = %s", self.ready)
+				self.recalculate(fetch_stale_interfaces = False)
+				debug("Recalculated: ready = %s; %d stale feeds", self.ready, len(self.stale_feeds))
 				if not self.ready: return False
 				self.start_downloading_impls()
 			except NeedDownload:
@@ -63,17 +67,17 @@ class AutoPolicy(policy.Policy):
 			raise NeedDownload(download_source.url)
 		return policy.Policy.begin_archive_download(self, download_source, success_callback, force = force)
 
-	def execute(self, prog_args, main = None):
+	def execute(self, prog_args, main = None, wrapper = None):
 		self.start_downloading_impls()
 		self.handler.wait_for_downloads()
 		if not self.download_only:
-			run.execute(self, prog_args, dry_run = self.dry_run, main = main)
+			run.execute(self, prog_args, dry_run = self.dry_run, main = main, wrapper = wrapper)
 		else:
 			info("Downloads done (download-only mode)")
 	
 	def recalculate_with_dl(self):
 		self.recalculate()
-		if self.handler.monitored_downloads:
+		while self.handler.monitored_downloads:
 			self.handler.wait_for_downloads()
 			self.recalculate()
 	
