@@ -23,7 +23,7 @@ class Signature:
 	def __init__(self, status):
 		self.status = status
 
-	def is_trusted(self):
+	def is_trusted(self, domain = None):
 		return False
 	
 	def need_key(self):
@@ -38,8 +38,8 @@ class ValidSig(Signature):
 	def __str__(self):
 		return "Valid signature from " + self.status[self.FINGERPRINT]
 	
-	def is_trusted(self):
-		return trust_db.is_trusted(self.status[self.FINGERPRINT])
+	def is_trusted(self, domain = None):
+		return trust_db.is_trusted(self.status[self.FINGERPRINT], domain)
 	
 	def get_timestamp(self):
 		return int(self.status[self.TIMESTAMP])
@@ -86,6 +86,59 @@ class ErrSig(Signature):
 			return self.status[self.KEYID]
 		return None
 
+class Key:
+	"""A GPG key.
+	@since: 0.27
+	"""
+	def __init__(self, fingerprint):
+		self.fingerprint = fingerprint
+		self.name = '(unknown)'
+	
+	def get_short_name(self):
+		return self.name.split(' (', 1)[0].split(' <', 1)[0]
+
+def load_keys(fingerprints):
+	"""Load a set of keys at once.
+	This is much more efficient than making individual calls to L{load_key}.
+	@return: a list of loaded keys, indexed by fingerprint
+	@rtype: {str: L{Key}}
+	@since 0.27"""
+
+	keys = {}
+
+	# Otherwise GnuPG returns everything...
+	if not fingerprints: return keys
+
+	for fp in fingerprints:
+		keys[fp] = Key(fp)
+
+	current_fpr = None
+
+	cin, cout = os.popen2(['gpg', '--fixed-list-mode', '--with-colons', '--list-keys', '--with-fingerprint'] + fingerprints)
+	cin.close()
+	try:
+		for line in cout:
+			if line.startswith('pub:'):
+				current_fpr = None
+			if line.startswith('fpr:'):
+				assert current_fpr is None
+				current_fpr = line.split(':')[9]
+			if line.startswith('uid:'):
+				assert current_fpr is not None
+				parts = line.split(':')
+				keys[current_fpr].name = parts[9]
+	finally:
+		cout.close()
+
+	return keys
+
+def load_key(fingerprint):
+	"""Query gpg for information about this key.
+	@return: a new key
+	@rtype: L{Key}
+	@since: 0.27"""
+	return load_keys([fingerprint])[fingerprint]
+
 def import_key(stream):
 	"""Run C{gpg --import} with this stream as stdin."""
 	errors = tempfile.TemporaryFile()
@@ -241,8 +294,8 @@ def check_stream(stream):
 	if not _find_in_path('gpg'):
 		raise SafeException("GnuPG is not installed ('gpg' not in $PATH). See http://gnupg.org")
 
-	stream.seek(0)
-	all = stream.read()
+	#stream.seek(0)
+	#all = stream.read()
 	stream.seek(0)
 
 	start = stream.read(6)
