@@ -9,7 +9,7 @@ import os
 from logging import debug, info, warn
 
 from zeroinstall.injector import basedir
-from zeroinstall import SafeException
+from zeroinstall import SafeException, support
 
 class BadDigest(SafeException):
 	"""Thrown if a digest is invalid (either syntactically or cryptographically)."""
@@ -40,17 +40,6 @@ def _copytree2(src, dst):
 		else:
 			shutil.copy2(srcname, dstname)
 
-def _wrap_umask(fn):
-	def wrapper(self, *args, **kwargs):
-		if self.public:
-			old_umask = os.umask(0022)	# World readable
-		try:
-			fn(self, *args, **kwargs)
-		finally:
-			if self.public:
-				os.umask(old_umask)
-	return wrapper
-
 class Store:
 	"""A directory for storing implementations."""
 
@@ -79,17 +68,18 @@ class Store:
 		"""Create a temporary directory in the directory where we would store an implementation
 		with the given digest. This is used to setup a new implementation before being renamed if
 		it turns out OK.
-		@raise: NonwritableStore if we can't create it"""
+		@raise NonwritableStore: if we can't create it"""
 		try:
 			if not os.path.isdir(self.dir):
 				os.makedirs(self.dir)
 			from tempfile import mkdtemp
 			tmp = mkdtemp(dir = self.dir, prefix = 'tmp-')
+			os.chmod(tmp, 0755)	# r-x for all; needed by 0store-helper
 			return tmp
 		except OSError, ex:
 			raise NonwritableStore(str(ex))
 	
-	def add_archive_to_cache(self, required_digest, data, url, extract = None, type = None, start_offset = 0):
+	def add_archive_to_cache(self, required_digest, data, url, extract = None, type = None, start_offset = 0, try_helper = False):
 		import unpack
 		info("Caching new implementation (digest %s)", required_digest)
 
@@ -106,23 +96,27 @@ class Store:
 			raise
 
 		try:
-			self.check_manifest_and_rename(required_digest, tmp, extract)
+			self.check_manifest_and_rename(required_digest, tmp, extract, try_helper = try_helper)
 		except Exception, ex:
 			warn("Leaving extracted directory as %s", tmp)
 			raise
-	add_archive_to_cache = _wrap_umask(add_archive_to_cache)
 	
-	def add_dir_to_cache(self, required_digest, path):
+	def add_dir_to_cache(self, required_digest, path, try_helper = False):
 		"""Copy the contents of path to the cache.
 		@param required_digest: the expected digest
 		@type required_digest: str
 		@param path: the root of the tree to copy
 		@type path: str
+		@param try_helper: attempt to use privileged helper before user cache (since 0.26)
+		@type try_helper: bool
 		@raise BadDigest: if the contents don't match the given digest."""
 		if self.lookup(required_digest):
 			info("Not adding %s as it already exists!", required_digest)
 			return
 
+		if try_helper and self._add_with_helper(required_digest, path):
+			return
+
 		tmp = self.get_tmp_dir_for(required_digest)
 		try:
 			_copytree2(path, tmp)
@@ -130,17 +124,37 @@ class Store:
 		except:
 			warn("Error importing directory.")
 			warn("Deleting %s", tmp)
-			import shutil
-			shutil.rmtree(tmp)
+			support.ro_rmtree(tmp)
 			raise
-	add_dir_to_cache = _wrap_umask(add_dir_to_cache)
 
-	def check_manifest_and_rename(self, required_digest, tmp, extract = None):
+	def _add_with_helper(self, required_digest, path):
+		"""Use 0store-helper to copy 'path' to the system store.
+		@param required_digest: the digest for path
+		@type required_digest: str
+		@param path: root of implementation directory structure
+		@type path: str
+		@return: True iff the directory was copied into the system cache successfully
+		"""
+		helper = support.find_in_path('0store-helper')
+		if not helper:
+			info("Command '0store-helper' not found in $PATH. Not importing to system store.")
+			return False
+
+		info("Trying to add to system cache using '%s'", helper)
+		if os.spawnv(os.P_WAIT, helper, [helper, required_digest, path]):
+			warn("0store-helper failed.")
+			return False
+
+		info("Added succcessfully.")
+		return True
+
+	def check_manifest_and_rename(self, required_digest, tmp, extract = None, try_helper = False):
 		"""Check that tmp[/extract] has the required_digest.
 		On success, rename the checked directory to the digest and,
 		if self.public, make the whole tree read-only.
+		@param try_helper: attempt to use privileged helper to import to system cache first (since 0.26)
+		@type try_helper: bool
 		@raise BadDigest: if the input directory doesn't match the given digest"""
-		# Should we make private stores read-only too?
 		if extract:
 			extracted = os.path.join(tmp, extract)
 			if not os.path.isdir(extracted):
@@ -149,6 +163,14 @@ class Store:
 			extracted = tmp
 
 		import manifest
+
+		manifest.fixup_permissions(extracted)
+		if try_helper:
+			if self._add_with_helper(required_digest, extracted):
+				support.ro_rmtree(tmp)
+				return
+			info("Can't add to system store. Trying user store instead.")
+
 		alg, required_value = manifest.splitID(required_digest)
 		actual_digest = alg.getID(manifest.add_manifest_file(extracted, alg))
 		if actual_digest != required_digest:
@@ -160,21 +182,16 @@ class Store:
 		final_name = os.path.join(self.dir, required_digest)
 		if os.path.isdir(final_name):
 			raise Exception("Item %s already stored." % final_name)
+
+		# If we just want a subdirectory then the rename will change
+		# extracted/.. and so we'll need write permission on 'extracted'
+
+		os.chmod(extracted, 0755)
+		os.rename(extracted, final_name)
+		os.chmod(final_name, 0555)
+
 		if extract:
-			os.rename(os.path.join(tmp, extract), final_name)
 			os.rmdir(tmp)
-		else:
-			os.rename(tmp, final_name)
-
-		if self.public:
-			import stat
-			for dirpath, dirnames, filenames in os.walk(final_name):
-				for item in ['.'] + filenames:
-					full = os.path.join(dirpath, item)
-					info = os.lstat(full)
-					if not stat.S_ISLNK(info.st_mode):
-						os.chmod(full, info.st_mode & ~stat.S_IWRITE)
-
 
 class Stores(object):
 	"""A list of L{Store}s. All stores are searched when looking for an implementation.
@@ -214,13 +231,13 @@ class Stores(object):
 	def add_dir_to_cache(self, required_digest, dir):
 		"""Add to the best writable cache.
 		@see: L{Store.add_dir_to_cache}"""
-		self._write_store(lambda store: store.add_dir_to_cache(required_digest, dir))
+		self._write_store(lambda store, **kwargs: store.add_dir_to_cache(required_digest, dir, **kwargs))
 
 	def add_archive_to_cache(self, required_digest, data, url, extract = None, type = None, start_offset = 0):
 		"""Add to the best writable cache.
 		@see: L{Store.add_archive_to_cache}"""
-		self._write_store(lambda store: store.add_archive_to_cache(required_digest,
-						data, url, extract, type = type, start_offset = start_offset))
+		self._write_store(lambda store, **kwargs: store.add_archive_to_cache(required_digest,
+						data, url, extract, type = type, start_offset = start_offset, **kwargs))
 	
 	def _write_store(self, fn):
 		"""Call fn(first_system_store). If it's read-only, try again with the user store."""
@@ -229,6 +246,6 @@ class Stores(object):
 				fn(self.stores[1])
 				return
 			except NonwritableStore:
-				debug("%s not-writable. Using user store instead.", self.stores[1])
+				debug("%s not-writable. Trying helper instead.", self.stores[1])
 				pass
-		fn(self.stores[0])
+		fn(self.stores[0], try_helper = True)
